function [u2,p1,u1,p2,t] = boundaryvaluesic(u1,p2,t,ts,L,c,I_u1,I_u2,I_v1,I_v2,gap)
%This function uses the Boundary equations to calculate the unknown BCs
%Switch in BC occurs when contact is activated
array_length = length(t);
delta_t = t(2) - t(1); var = L/c;
u2 = zeros(array_length,1);%u2 is the displacement at u(L,t)
p1 = zeros(array_length,1);%p1 is the flux at p(0,t)
for i = 1:array_length
    if i==1
        u2(i) = 0 + 2*I_u2(i,1) + 2*I_v2(i,1);
    elseif i > 1 && t(i) < var
        p2sum = 0; %initialise
        for j = 1:i-1
            p2sum = p2sum + p2(j);
        end
        u2(i) = (p2sum)*(c*delta_t) + 2*I_u2(i,1) + 2*I_v2(i,1); % u(L,t(i))
        p1sum = 0; %initialise
        for j = 1:i-2
            p1sum = p1sum + p1(j);
        end
        p1(i-1) = (u1(i))/(c*delta_t) - p1sum - 2*I_u1(i,1)/(c*delta_t) - 2*I_v1(i,1)/(c*delta_t); % p(0, t(i-1))
    elseif t(i) == var
        p2sum = 0; %initialise
        for j = 1:i-1
            p2sum = p2sum + p2(j);
        end
        u2(i) = u1(i-ts) + p2sum*c*delta_t + 2*I_u2(i,1) + 2*I_v2(i,1);
        p1sum = 0; %initialise
        for j = 1:i-2
            p1sum = p1sum + p1(j);
        end
        p1(i-1) = (u1(i) - u2(i-ts))/(c*delta_t) - p1sum - 2*I_u1(i,1)/(c*delta_t) - 2*I_v1(i,1)/(c*delta_t);
    elseif t(i) > var && t(i) <= t(array_length)
        p1sum = 0; p2sum = 0; %initialise
        for j = 1:i-(ts+1)
            p1sum = p1sum + p1(j);
        end
        for j = 1:i-1
            p2sum = p2sum + p2(j);
        end
        u2(i) = u1(i-ts) + (p1sum + p2sum)*(c*delta_t) + 2*I_u2(i,1) + 2*I_v2(i,1);
        p1sum = 0; p2sum = 0; %initialise
        for j = 1:i-2
            p1sum = p1sum + p1(j);
        end
        for j = 1:i-(ts+1)
            p2sum = p2sum + p2(j);
        end
        p1(i-1) = (u1(i) - u2(i-ts))/(c*delta_t) - p1sum - p2sum - 2*I_u1(i,1)/(c*delta_t) - 2*I_v1(i,1)/(c*delta_t);
    end
    p1sum = 0; p2sum = 0; %initialise
    for j = 1:array_length-1
        p1sum = p1sum + p1(j);
    end
    for j = 1:array_length-(ts)
        p2sum = p2sum + p2(j);
    end
    p1(array_length) = (u1(array_length) - u2(array_length+1-ts))/(c*delta_t) - p1sum - p2sum - 2*I_u1(array_length,1)/(c*delta_t) - 2*I_v1(array_length,1)/(c*delta_t);

    if u2(i) >= gap % Check for contact or penetration
        u2(i) = gap; % Fix contacting end to the wall
        if i==1
            u1(i) = 0;
        elseif i > 1 && t(i) < var
            p1sum = 0; %initialise
            for j = 1:i-2
                p1sum = p1sum + p1(j);
            end
            p1(i-1) = (u1(i))/(c*delta_t) - p1sum - 2*I_u1(i,1)/(c*delta_t) - 2*I_v1(i,1)/(c*delta_t); %
            
            p2sum = 0; %initialise
            for j = 1:i-2
                p2sum = p2sum + p2(j);
            end
            p2(i-1) = (u2(i))/(c*delta_t) - p2sum - 2*I_u2(i,1)/(c*delta_t) - 2*I_v2(i,1)/(c*delta_t); % p(0, t(i-1))
        elseif t(i) == var
            p1sum = 0; %initialise
            for j = 1:i-2
                p1sum = p1sum + p1(j);
            end
            p1(i-1) = (u1(i) - u2(i-ts))/(c*delta_t) - p1sum - 2*I_u1(i,1)/(c*delta_t) - 2*I_v1(i,1)/(c*delta_t);
            p2sum = 0; %initialise
            for j = 1:i-2
                p2sum = p2sum + p2(j);
            end
            p2(i-1) = (u2(i) - u1(i-ts))/(c*delta_t) - p2sum - 2*I_u2(i,1)/(c*delta_t) - 2*I_v2(i,1)/(c*delta_t);
        elseif t(i) > var && t(i) <= t(array_length)
            p1sum = 0; p2sum = 0; %initialise
            for j = 1:i-2
                p1sum = p1sum + p1(j);
            end
            for j = 1:i-(ts+1)
                p2sum = p2sum + p2(j);
            end
            p1(i-1) = (u1(i) - u2(i-ts))/(c*delta_t) - p1sum - p2sum - 2*I_u1(i,1)/(c*delta_t) - 2*I_v1(i,1)/(c*delta_t);
            p1sum = 0; p2sum = 0; %initialise
            for j = 1:i-2
                p2sum = p2sum + p2(j);
            end
            for j = 1:i-(ts+1)
                p1sum = p1sum + p1(j);
            end
            p2(i-1) = (u2(i) - u1(i-ts))/(c*delta_t) - p1sum - p2sum - 2*I_u2(i,1)/(c*delta_t) - 2*I_v2(i,1)/(c*delta_t);
        end
        if u2(array_length) >= gap
            p1sum = 0; p2sum = 0; %initialise
            for j = 1:array_length-1
                p1sum = p1sum + p1(j);
            end
            for j = 1:array_length-(ts)
                p2sum = p2sum + p2(j);
            end
            p1(array_length) = (u1(array_length) - u2(array_length+1-ts))/(c*delta_t) - p1sum - p2sum - 2*I_u1(array_length,1)/(c*delta_t) - 2*I_v1(array_length,1)/(c*delta_t);
            p1sum = 0; p2sum = 0; %initialise
            for j = 1:array_length-1
                p2sum = p2sum + p2(j);
            end
            for j = 1:array_length-(ts)
                p1sum = p1sum + p1(j);
            end
            p2(array_length) = (u2(array_length) - u1(array_length+1-ts))/(c*delta_t) - p1sum - p2sum - 2*I_u2(array_length,1)/(c*delta_t) - 2*I_v2(array_length,1)/(c*delta_t);
        end
        if i > 1
            if p2(i-1) >= 2.2204e-14 % Check if contact force is negative
                break
            end
        end
    end
end
end
